function Enemy(pName,pX,pY,pWidth,pHeight){
	var e = new Entity("enemy_"+pName,pX,pY,pWidth,pHeight);
	var canShoot = false,
		attackCD = 300,
		lastAttack = 0,
		dropChances = [],
		drops = [],
		preventDrop = false,
		startingHealth = 1,
		manaWorth = 1;
	e.SetFaction(1); //Enemy Faction
	e.SetBulletSize(5);
	e.SetBulletColor("rgba(255,0,0,1)");

	e.CanShoot = function(){
		return canShoot;
	};

	e.PreventDrops = function(){
		preventDrop = true;
	}

	e.AllowShoot = function(){
		canShoot = true;
	};

	e.DenyShoot = function(){
		canShoot = false;
	};

	e.GetDrops = function(){
		return drops;
	};

	e.AddDrop = function(pChance,pPowerup){
		//Chance is percentage of 0-1
		dropChances.push(pChance);
		drops.push(pPowerup);
	};

	e.ClearDrops = function(){
		drops = [];
		dropChances = [];
	};

	e.SetStartingHealth = function(pStartingHealth){
		startingHealth = pStartingHealth;
		this.SetHealth(pStartingHealth);
	};

	e.SetManaWorth = function(pManaWorth){
		manaWorth = pManaWorth;
	};

	e.GetStartingHealth = function(){
		return startingHealth;
	};

	e.DropObject = function(){
		if(preventDrop)
			return;

		var dropped = false;
		for(var i in drops){
			if(Math.floor(Math.random()*100) <= Math.floor(dropChances[i]*Lynx.Constants.DROP_MOD) && !dropped){
				var drop = drops[i];
					drop.SetX(this.GetX());
					drop.SetY(this.GetY());
					drop.SetName(drop.GetType()+"_"+drop.GetX()+"_"+drop.GetY()+"_"+Lynx.Time.GetDelta());
				Lynx.Game.GetCurrentScene().AddObject(drop);
				dropped = true;
			}
		}
		if(!dropped){
			//Drop mana
			if(manaWorth > 0){
				Lynx.Game.GetCurrentScene().AddObject(new Powerup(this.GetX(),this.GetY(),PowerupType.MANA_UP,manaWorth));
			}
		}
	};

	e.onSpawn = function(){
		//For special mechanics
	};

	e.onDeath = function(){
		this.DropObject();
		Lynx.Constants.CURRENT_SCORE = Lynx.Constants.CURRENT_SCORE + (this.GetDamage() + this.GetStartingHealth())*Lynx.Constants.MULTIPLIER;
		this.Remove();
	};

	e.Internal = {
		onDeath: function(object){
			object.DropObject();
			Lynx.Constants.CURRENT_SCORE = Lynx.Constants.CURRENT_SCORE + (object.GetDamage() + object.GetStartingHealth())*Lynx.Constants.MULTIPLIER;
			object.Remove();
		}
	};

	e.Update = function(){
		//This is where the enemy AI Goes. By default only chases player.
		if(this.WillRemove()){
			this.RemoveObject();
			return;
		}
		var player = Lynx.Constants.PLAYER;
		if(canShoot){
			this.Shoot(player.GetX()+Math.floor(player.GetWidth()/2),player.GetY()+Math.floor(player.GetHeight()/2));
		}
		var dX = player.GetX() - (this.GetX() + this.GetWidth()/2);
		var dY = player.GetY() - (this.GetY() + this.GetHeight()/2);
		var mag = Math.sqrt(dX*dX + dY*dY);
		this.Move((this.GetTimedSpeed()/mag)*dX,(this.GetTimedSpeed()/mag)*dY);
	};

	e.CanAttack = function(){
		return (Date.now() - lastAttack > attackCD);
	};

	e.oncollision = function(pEvent){
		if(pEvent.collidingObject.GetName() == "player"){
			if(this.CanAttack()){
				pEvent.collidingObject.Hurt(this.GetDamage());
				lastAttack = Date.now();
			}
		}
		pEvent.resolved = true;
	};

	e.SetCollisions(true);


	return e;
};